from django import forms
from edc_form_validators import FormValidatorMixin
from edc_sites.forms import SiteModelFormMixin

from ...form_validators import DeathReportTmgFormValidator


class DeathReportTmgModelFormMixin(
    SiteModelFormMixin,
    FormValidatorMixin,
):

    form_validator_cls = DeathReportTmgFormValidator

    class Meta:
        help_text = {"subject_identifier": "(read-only)", "action_identifier": "(read-only)"}
        widgets = {
            "subject_identifier": forms.TextInput(attrs={"readonly": "readonly"}),
            "action_identifier": forms.TextInput(attrs={"readonly": "readonly"}),
        }



from django.apps import apps as django_apps
from django.core.exceptions import ObjectDoesNotExist
from django.utils.html import format_html
from edc_action_item.model_wrappers import (
    ActionItemModelWrapper as BaseActionItemModelWrapper,
)
from edc_constants.constants import CLOSED, NEW, OPEN
from edc_dashboard.view_mixins import (
    EdcViewMixin,
    ListboardFilterViewMixin,
    SearchFormViewMixin,
)
from edc_dashboard.views import ListboardView as BaseListboardView
from edc_navbar import NavbarViewMixin
from edc_utils import get_utcnow

from ...constants import AE_INITIAL_ACTION
from ...get_ae_model import get_ae_model
from ...model_wrappers import AeInitialModelWrapper, DeathReportModelWrapper
from ...pdf_reports import AeReport


class ActionItemModelWrapper(BaseActionItemModelWrapper):

    ae_initial_model_wrapper = AeInitialModelWrapper
    death_report_model_wrapper = DeathReportModelWrapper
    next_url_name = "ae_listboard_url"

    def __init__(self, model_obj=None, **kwargs):
        self._death_report = None
        super().__init__(model_obj=model_obj, **kwargs)

    @property
    def death_report(self):
        if not self._death_report:
            model_cls = django_apps.get_model(self.death_report_model_wrapper.model)
            try:
                self._death_report = self.death_report_model_wrapper(
                    model_obj=model_cls.objects.get(subject_identifier=self.subject_identifier)
                )
            except ObjectDoesNotExist:
                self._death_report = None
        return self._death_report

    @property
    def ae_initial(self):
        return self.ae_initial_model_wrapper(model_obj=self.object.reference_obj)


class AeListboardViewMixin(
    NavbarViewMixin,
    EdcViewMixin,
    ListboardFilterViewMixin,
    SearchFormViewMixin,
    BaseListboardView,
):

    pdf_report_cls = AeReport

    listboard_back_url = "ae_home_url"
    home_url = "ae_home_url"
    listboard_panel_title = "Adverse Events: AE Initial and Follow-up Reports"
    model_wrapper_cls = ActionItemModelWrapper

    listboard_template = "ae_listboard_template"
    listboard_url = "ae_listboard_url"
    listboard_panel_style = "default"
    listboard_model = "edc_action_item.actionitem"
    listboard_view_permission_codename = "edc_dashboard.view_ae_listboard"

    listboard_instructions = format_html(
        (
            "To find an initial adverse event report, search on the subject's "
            "study identifier or AE reference number."
        )
        + " <BR>"
        + "To download the printable report, click on the PDF button"
        + " <i class='fas fa-file-pdf fa-fw'></i> "
        + "left of the subject's identifier."
    )
    navbar_selected_item = "ae_home"
    ordering = "-report_datetime"
    paginate_by = 25
    search_form_url = "ae_listboard_url"
    action_type_names = [AE_INITIAL_ACTION]

    search_fields = [
        "subject_identifier",
        "action_identifier",
        "parent_action_item__action_identifier",
        "related_action_item__action_identifier",
        "user_created",
        "user_modified",
    ]

    def get(self, request, *args, **kwargs):
        if request.GET.get("pdf"):
            response = self.print_pdf_report(
                action_identifier=self.request.GET.get("pdf"), request=request
            )
            return response or super().get(request, *args, **kwargs)
        return super().get(request, *args, **kwargs)

    def print_pdf_report(self, action_identifier=None, request=None):
        try:
            ae_initial = get_ae_model("aeinitial").objects.get(
                action_identifier=action_identifier
            )
        except ObjectDoesNotExist:
            pass
        else:
            report = self.pdf_report_cls(
                ae_initial=ae_initial,
                subject_identifier=ae_initial.subject_identifier,
                user=self.request.user,
                request=request,
            )
            return report.render()
        return None

    def get_context_data(self, **kwargs) -> dict:
        context = super().get_context_data(**kwargs)
        context = self.add_url_to_context(
            new_key="ae_home_url", existing_key=self.home_url, context=context
        )
        context["AE_INITIAL_ACTION"] = AE_INITIAL_ACTION
        context["utc_date"] = get_utcnow().date()
        return context

    def get_queryset_filter_options(self, request, *args, **kwargs):
        options = super().get_queryset_filter_options(request, *args, **kwargs)
        options.update(
            action_type__name__in=self.action_type_names, status__in=[NEW, OPEN, CLOSED]
        )
        if kwargs.get("subject_identifier"):
            options.update({"subject_identifier": kwargs.get("subject_identifier")})
        return options

    def get_updated_queryset(self, queryset):
        pks = []
        for obj in queryset:
            try:
                obj.reference_obj
            except ObjectDoesNotExist:
                pks.append(obj.pk)
        return queryset.exclude(pk__in=pks)



from django import forms
from edc_constants.constants import CLOSED, NO
from edc_form_validators import FormValidator, FormValidatorMixin
from edc_registration.modelform_mixins import ModelFormSubjectIdentifierMixin


class DefaultAeTmgFormValidator(FormValidator):
    def clean(self):

        self.validate_other_specify(field="ae_classification")

        self.required_if(NO, field="original_report_agreed", field_required="narrative")

        self.required_if(
            CLOSED, field="report_status", field_required="report_closed_datetime"
        )


class AeTmgModelFormMixin(FormValidatorMixin, ModelFormSubjectIdentifierMixin):

    form_validator_cls = DefaultAeTmgFormValidator

    class Meta:
        labels = {
            "ae_description": "Original AE Description",
            "ae_classification": "AE Classification",
            "ae_classification_other": "AE Classification (if `other` above)",
        }
        help_text = {
            "subject_identifier": "(read-only)",
            "action_identifier": "(read-only)",
            "ae_description": "(read-only)",
            "ae_classification": "(read-only)",
            "ae_classification_other": "(read-only)",
        }
        widgets = {
            "subject_identifier": forms.TextInput(attrs={"readonly": "readonly"}),
            "action_identifier": forms.TextInput(attrs={"readonly": "readonly"}),
            "ae_description": forms.TextInput(attrs={"readonly": "readonly", "cols": "79"}),
            "ae_classification": forms.TextInput(attrs={"readonly": "readonly"}),
            "ae_classification_other": forms.TextInput(attrs={"readonly": "readonly"}),
        }



from django.apps import apps as django_apps
from django.conf import settings


def get_ae_model(model_name):
    return django_apps.get_model(f"{settings.ADVERSE_EVENT_APP_LABEL}.{model_name}")


def get_ae_model_name(model_name):
    return f"{settings.ADVERSE_EVENT_APP_LABEL}.{model_name}"



from django.contrib import admin
from edc_list_data.admin import ListModelAdminMixin

from .admin_site import edc_adverse_event_admin
from .models import AeClassification, CauseOfDeath, SaeReason


@admin.register(AeClassification, site=edc_adverse_event_admin)
class AeClassificationAdmin(ListModelAdminMixin, admin.ModelAdmin):
    pass


@admin.register(CauseOfDeath, site=edc_adverse_event_admin)
class CauseOfDeathAdmin(ListModelAdminMixin, admin.ModelAdmin):
    pass


@admin.register(SaeReason, site=edc_adverse_event_admin)
class SaeReasonAdmin(ListModelAdminMixin, admin.ModelAdmin):
    pass



from .ae_initial_model_wrapper import AeInitialModelWrapper
from .ae_tmg_model_wrapper import AeTmgModelWrapper
from .death_report_model_wrapper import DeathReportModelWrapper
from .death_report_tmg_model_wrapper import DeathReportTmgModelWrapper
from .death_report_tmg_second_model_wrapper import DeathReportTmgSecondModelWrapper
from .tmg import (
    ClosedTmgActionItemModelWrapper,
    NewTmgActionItemModelWrapper,
    OpenTmgActionItemModelWrapper,
    TmgActionItemModelWrapper,
)



from django import forms
from django.conf import settings
from edc_constants.constants import DEAD, YES
from edc_form_validators import FormValidator, FormValidatorMixin
from edc_registration.modelform_mixins import ModelFormSubjectIdentifierMixin
from edc_reportable import GRADE5, SEVERITY_INCREASED_FROM_G3
from edc_utils.text import convert_php_dateformat


def validate_ae_initial_outcome_date(form_obj):
    ae_initial = form_obj.cleaned_data.get("ae_initial")
    if not ae_initial and form_obj.instance:
        ae_initial = form_obj.instance.ae_initial
    outcome_date = form_obj.cleaned_data.get("outcome_date")
    if ae_initial and outcome_date:
        if outcome_date < ae_initial.ae_start_date:
            formatted_dte = ae_initial.ae_start_date.strftime(
                convert_php_dateformat(settings.SHORT_DATE_FORMAT)
            )
            raise forms.ValidationError(
                {"outcome_date": f"May not be before the AE start date {formatted_dte}."}
            )


class DefaultAeFollowupFormValidator(FormValidator):
    def clean(self):

        self.applicable_if(
            SEVERITY_INCREASED_FROM_G3, field="outcome", field_applicable="ae_grade"
        )

        self.applicable_if(
            SEVERITY_INCREASED_FROM_G3, field="outcome", field_applicable="ae_grade"
        )


class AeFollowupModelFormMixin(FormValidatorMixin, ModelFormSubjectIdentifierMixin):

    form_validator_cls = DefaultAeFollowupFormValidator

    def clean(self):
        cleaned_data = super().clean()

        validate_ae_initial_outcome_date(self)
        self.validate_no_followup_upon_death()

        return cleaned_data

    def validate_no_followup_upon_death(self):
        if self.cleaned_data.get("followup") == YES:
            if (
                self.cleaned_data.get("ae_grade") == GRADE5
                or self.cleaned_data.get("outcome") == DEAD
            ):
                raise forms.ValidationError(
                    {
                        "followup": (
                            "Expected No. Submit a death report when the "
                            "severity increases to grade 5."
                        )
                    }
                )

    class Meta:
        help_text = {"subject_identifier": "(read-only)", "action_identifier": "(read-only)"}
        widgets = {
            "subject_identifier": forms.TextInput(attrs={"readonly": "readonly"}),
            "action_identifier": forms.TextInput(attrs={"readonly": "readonly"}),
        }



from typing import Tuple

from django.conf import settings
from django.contrib import admin
from django.core.exceptions import ObjectDoesNotExist
from django.template.loader import render_to_string
from django.urls.base import reverse
from django.utils.html import format_html
from django.utils.safestring import mark_safe
from django_audit_fields.admin import audit_fieldset_tuple
from edc_action_item import action_fieldset_tuple
from edc_action_item.modeladmin_mixins import ActionItemModelAdminMixin
from edc_constants.constants import DEAD
from edc_model_admin.dashboard import ModelAdminSubjectDashboardMixin
from edc_notification.utils import get_email_contacts

from ..forms import AeInitialForm
from ..get_ae_model import get_ae_model
from ..templatetags.edc_adverse_event_extras import (
    format_ae_description,
    select_description_template,
)
from .modeladmin_mixins import AdverseEventModelAdminMixin

fieldset_part_one = (
    "Part 1: Description",
    {
        "fields": (
            "ae_classification",
            "ae_classification_other",
            "ae_description",
            "ae_awareness_date",
            "ae_start_date",
            "ae_grade",
        )
    },
)

fieldset_part_two = (
    "Part 2: Cause and relationship to study",
    {
        "fields": (
            "ae_study_relation_possibility",
            "study_drug_relation",
            "ae_cause",
            "ae_cause_other",
        )
    },
)

fieldset_part_three = ("Part 3: Treatment", {"fields": ("ae_treatment",)})

fieldset_part_four = (
    "Part 4: SAE / SUSAR",
    {"fields": ("sae", "sae_reason", "susar", "susar_reported")},
)

default_radio_fields = {
    "ae_cause": admin.VERTICAL,
    "ae_classification": admin.VERTICAL,
    "ae_grade": admin.VERTICAL,
    "ae_study_relation_possibility": admin.VERTICAL,
    "study_drug_relation": admin.VERTICAL,
    "sae": admin.VERTICAL,
    "sae_reason": admin.VERTICAL,
    "susar": admin.VERTICAL,
    "susar_reported": admin.VERTICAL,
}


class AeInitialModelAdminMixin(
    AdverseEventModelAdminMixin,
    ModelAdminSubjectDashboardMixin,
    ActionItemModelAdminMixin,
):

    form = AeInitialForm

    email_contact = get_email_contacts("ae_reports")
    additional_instructions = format_html(  # nosec B308, B703
        "Complete the initial AE report and forward to the TMG. "
        'Email to <a href="mailto:{}">{}</a>',
        mark_safe(email_contact),  # nosec B308, B703
        mark_safe(email_contact),  # nosec B308, B703
    )

    fieldsets = (
        (None, {"fields": ("subject_identifier", "report_datetime")}),
        fieldset_part_one,
        fieldset_part_two,
        fieldset_part_three,
        fieldset_part_four,
        action_fieldset_tuple,
        audit_fieldset_tuple,
    )

    radio_fields = default_radio_fields

    ordering = ("-tracking_identifier",)

    search_fields = ("subject_identifier", "action_identifier", "tracking_identifier")

    def get_list_display(self, request) -> Tuple[str, ...]:
        list_display = super().get_list_display(request)
        custom_fields = (
            "identifier",
            "dashboard",
            "description",
            "follow_up_reports",
            "user",
        )
        return custom_fields + tuple(f for f in list_display if f not in custom_fields)

    def get_list_filter(self, request) -> Tuple[str, ...]:
        list_filter = super().get_list_filter(request)
        custom_fields = (
            "ae_awareness_date",
            "ae_grade",
            "ae_classification",
            "sae",
            "sae_reason",
            "susar",
            "susar_reported",
        )
        return custom_fields + tuple(f for f in list_filter if f not in custom_fields)

    def if_sae_reason(self, obj):
        """Returns the SAE reason.

        If DEATH, adds link to the death report.
        """
        if obj.sae_reason.name == DEAD:
            death_report_cls = get_ae_model("deathreport")
            try:
                death_report = death_report_cls.objects.get(
                    subject_identifier=obj.subject_identifier
                )
            except ObjectDoesNotExist:
                link = '<font color="red">Death report pending</font>'
            else:
                url_name = f"{settings.ADVERSE_EVENT_APP_LABEL}_deathreport"
                namespace = self.admin_site.name
                url = reverse(f"{namespace}:{url_name}_changelist")
                link = format_html(  # nosec B308, B703
                    'See report <a title="go to Death report"'
                    'href="{}?q={}"><span nowrap>{}</span></a>',
                    mark_safe(url),  # nosec B308, B703
                    death_report.subject_identifier,
                    death_report.identifier,
                )
            return format_html(f"{obj.sae_reason.name}.<BR>{link}.")
        return obj.get_sae_reason_display()

    if_sae_reason.short_description = "If SAE, reason"

    @staticmethod
    def description(obj):
        """Returns a formatted comprehensive description of the SAE
        combining multiple fields.
        """
        context = format_ae_description({}, obj, 50)
        return render_to_string(select_description_template("aeinitial"), context)



from edc_constants.constants import CLOSED, NO
from edc_form_validators import FormValidator


class AeTmgFormValidator(FormValidator):
    def clean(self):

        self.validate_other_specify(field="ae_classification")

        self.required_if(NO, field="original_report_agreed", field_required="narrative")

        self.required_if(
            CLOSED, field="report_status", field_required="report_closed_datetime"
        )



from django import forms
from edc_form_validators import FormValidator, FormValidatorMixin
from edc_registration.modelform_mixins import ModelFormSubjectIdentifierMixin


class DefaultAeSusarFormValidator(FormValidator):
    pass


class AeSusarModelFormMixin(FormValidatorMixin, ModelFormSubjectIdentifierMixin):

    form_validator_cls = DefaultAeSusarFormValidator

    class Meta:
        help_text = {"subject_identifier": "(read-only)", "action_identifier": "(read-only)"}
        widgets = {
            "subject_identifier": forms.TextInput(attrs={"readonly": "readonly"}),
            "action_identifier": forms.TextInput(attrs={"readonly": "readonly"}),
        }



from .ae import (
    AE,
    AE_REVIEW,
    AE_ROLE,
    AE_SUPER,
    ae_codenames,
    ae_dashboard_tuples,
    ae_navbar_tuples,
)
from .tmg import (
    TMG,
    TMG_REVIEW,
    TMG_ROLE,
    tmg_codenames,
    tmg_dashboard_tuples,
    tmg_navbar_tuples,
    tmg_view_codenames,
)



from .ae_followup_form import AeFollowupForm
from .ae_initial_form import AeInitialForm
from .ae_susar_form import AeSusarForm
from .ae_tmg_form import AeTmgForm
from .death_report_form import DeathReportForm
from .death_report_tmg_form import DeathReportTmgForm
from .death_report_tmg_second_form import DeathReportTmgSecondForm
from .mixins import (
    AeFollowupModelFormMixin,
    AeInitialModelFormMixin,
    AeSusarModelFormMixin,
    AeTmgModelFormMixin,
    DeathReportModelFormMixin,
    DeathReportTmgModelFormMixin,
    validate_ae_initial_outcome_date,
)



from django.conf import settings
from django.db import models
from edc_model.validators import datetime_not_future
from edc_utils import get_utcnow


class AeSusarFieldsModelMixin(models.Model):

    ae_initial = models.ForeignKey(
        f"{settings.ADVERSE_EVENT_APP_LABEL}.aeinitial", on_delete=models.PROTECT
    )

    report_datetime = models.DateTimeField(
        verbose_name="Report date and time",
        validators=[datetime_not_future],
        default=get_utcnow,
    )

    submitted_datetime = models.DateTimeField(
        verbose_name="AE SUSAR submitted on",
        validators=[datetime_not_future],
        null=True,
        blank=True,
    )

    class Meta:
        abstract = True



from typing import Tuple

from django.contrib import admin
from django_audit_fields.admin import audit_fieldset_tuple
from edc_action_item import action_fieldset_tuple
from edc_action_item.modeladmin_mixins import ActionItemModelAdminMixin
from edc_model_admin.dashboard import ModelAdminSubjectDashboardMixin


class DeathReportModelAdminMixin(ModelAdminSubjectDashboardMixin, ActionItemModelAdminMixin):

    form = None

    fieldsets = (
        (
            None,
            {
                "fields": (
                    "subject_identifier",
                    "report_datetime",
                    "death_datetime",
                    "study_day",
                    "death_as_inpatient",
                )
            },
        ),
        (
            "Opinion of Local Study Doctor",
            {"fields": ("cause_of_death", "cause_of_death_other", "narrative")},
        ),
        action_fieldset_tuple,
        audit_fieldset_tuple,
    )

    radio_fields = {
        "death_as_inpatient": admin.VERTICAL,
        "cause_of_death": admin.VERTICAL,
    }

    def get_search_fields(self, request) -> Tuple[str, ...]:
        search_fields = super().get_search_fields(request)
        custom_fields = ("subject_identifier", "action_identifier", "tracking_identifier")
        return tuple(set(search_fields + custom_fields))

    def get_list_display(self, request) -> Tuple[str, ...]:
        list_display = super().get_list_display(request)
        custom_fields = (
            "subject_identifier",
            "dashboard",
            "report_datetime",
            "cause_of_death",
            "death_datetime",
            "action_item",
            "parent_action_item",
        )
        return custom_fields + tuple(f for f in list_display if f not in custom_fields)

    def get_list_filter(self, request) -> Tuple[str, ...]:
        list_filter = super().get_list_filter(request)
        custom_fields = ("report_datetime", "death_datetime", "cause_of_death")
        return custom_fields + tuple(f for f in list_filter if f not in custom_fields)



from django.db import models
from edc_action_item.managers import (
    ActionIdentifierManager,
    ActionIdentifierSiteManager,
)

from ...constants import DEATH_REPORT_TMG_SECOND_ACTION
from .death_report_tmg_model_mixin import DeathReportTmgModelMixin


class DeathReportTmgSecondManager(ActionIdentifierManager):
    def get_queryset(self):
        qs = super().get_queryset()
        return qs.filter(action_item__action_type__name=DEATH_REPORT_TMG_SECOND_ACTION)


class DeathReportTmgSecondSiteManager(ActionIdentifierSiteManager):
    def get_queryset(self):
        qs = super().get_queryset()
        return qs.filter(action_item__action_type__name=DEATH_REPORT_TMG_SECOND_ACTION)


class DeathReportTmgSecondModelMixin(DeathReportTmgModelMixin):

    action_name = DEATH_REPORT_TMG_SECOND_ACTION

    tracking_identifier_prefix = "DT"

    on_site = DeathReportTmgSecondSiteManager()

    objects = DeathReportTmgSecondManager()

    class Meta:
        abstract = True
        verbose_name = "Death Report TMG (2nd)"
        verbose_name_plural = "Death Report TMG (2nd)"
        indexes = [
            models.Index(fields=["subject_identifier", "action_identifier", "site", "id"])
        ]



import re

from django.conf import settings
from django.db.models import Q
from edc_dashboard.url_names import url_names
from edc_dashboard.view_mixins import (
    EdcViewMixin,
    ListboardFilterViewMixin,
    SearchFormViewMixin,
)
from edc_dashboard.views import ListboardView as BaseListboardView
from edc_navbar import NavbarViewMixin

from ...model_wrappers import DeathReportModelWrapper as ModelWrapper


class DeathReportModelWrapper(ModelWrapper):
    next_url_name = "tmg_death_listboard_url"


class DeathListboardView(
    NavbarViewMixin,
    EdcViewMixin,
    ListboardFilterViewMixin,
    SearchFormViewMixin,
    BaseListboardView,
):

    listboard_back_url = "tmg_home_url"

    listboard_template = "tmg_death_listboard_template"
    listboard_url = "tmg_death_listboard_url"
    listboard_panel_style = "warning"
    listboard_model = f"{settings.ADVERSE_EVENT_APP_LABEL}.deathreport"
    listboard_model_manager_name = "objects"
    listboard_panel_title = "TMG: Death Reports"
    listboard_view_permission_codename = "edc_dashboard.view_tmg_listboard"
    model_wrapper_cls = DeathReportModelWrapper
    navbar_selected_item = "tmg_home"
    ordering = "-created"
    paginate_by = 25
    search_form_url = "tmg_death_listboard_url"
    search_fields = [
        "subject_identifier",
        "action_identifier",
        "parent_action_item__action_identifier",
        "related_action_item__action_identifier",
        "user_created",
        "user_modified",
    ]

    def get_context_data(self, **kwargs):
        context = super().get_context_data(**kwargs)
        context.update({"subject_dashboard_url": url_names.get("subject_dashboard_url")})
        if self.kwargs.get("subject_identifier"):
            context.update({"q": self.kwargs.get("subject_identifier")})
        return context

    def get_queryset_filter_options(self, request, *args, **kwargs):
        options = super().get_queryset_filter_options(request, *args, **kwargs)
        if kwargs.get("subject_identifier"):
            options.update({"subject_identifier": kwargs.get("subject_identifier")})
        return options

    def extra_search_options(self, search_term):
        q = Q()
        if re.match("^[A-Z]+$", search_term):
            q = Q(first_name__exact=search_term)
        return [q]
